/* BEGIN LICENSE
 * Copyright © Blue Mind SAS, 2012-2016
 *
 * This file is part of BlueMind. BlueMind is a messaging and collaborative
 * solution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU Affero General Public License as
 * published by the Free Software Foundation (version 3 of the License).
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See LICENSE.txt
 * END LICENSE
 */
package net.bluemind.directory.api;

import java.util.Collection;
import java.util.Collections;

import net.bluemind.core.api.BMApi;
import net.bluemind.core.api.Email;
import net.bluemind.server.api.Server;

@BMApi(version = "3")
public class DirBaseValue {

	public String orgUnitUid;

	/**
	 * user emails
	 */
	public Collection<Email> emails = Collections.emptyList();

	/**
	 * true when this entry must not appear in user visible views
	 */
	public boolean hidden;

	/**
	 * true if the entry is suspended. A suspended entry cannot sign into the
	 * system but its data is still in the system.
	 */
	public boolean archived;

	/**
	 * true for some internal DirEntry used for specific internal tasks.
	 */
	public boolean system;

	/**
	 * Defines on which {@link Server} the data owned by this entry are.
	 */
	public String dataLocation;

	/**
	 * @return the default email for the user
	 */
	public Email defaultEmail() {
		if (emails == null) {
			return null;
		}

		Email ret = null;
		for (Email mail : emails) {
			if (mail.isDefault) {
				ret = mail;
				break;
			}
		}
		return ret;
	}

	public String defaultEmailAddress() {
		Email defaultEmail = defaultEmail();
		if (defaultEmail != null) {
			return defaultEmail.address;
		} else {
			return null;
		}
	}

	public String defaultEmailAddress(String domainName) {
		Email defaultEmail = defaultEmail();
		if (defaultEmail != null) {
			return defaultEmail.allAliases ? defaultEmail.localPart() + "@" + domainName : defaultEmail.address;
		} else {
			return null;
		}
	}
}
