/* BEGIN LICENSE
 * Copyright © Blue Mind SAS, 2012-2016
 *
 * This file is part of BlueMind. BlueMind is a messaging and collaborative
 * solution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU Affero General Public License as
 * published by the Free Software Foundation (version 3 of the License).
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See LICENSE.txt
 * END LICENSE
 */
package net.bluemind.directory.api;

import java.util.ArrayList;
import java.util.List;

import net.bluemind.core.api.BMApi;
import net.bluemind.core.api.Email;

/**
 * The vcard contains contact information, typically pertaining to a single
 * contact or group of contacts.
 */
@BMApi(version = "3")
public class DirEntry extends BaseDirEntry {

	public String path;
	public String email;
	public boolean hidden;
	public boolean system;
	public boolean archived;
	// FIXME not used for now
	public List<Email> emails;
	public String orgUnitUid;
	public OrgUnitPath orgUnitPath;
	public String dataLocation;

	// FIXME remove path parameter
	public static DirEntry create(String orgUnitUid, String path, Kind kind, String entryUid, String displayName,
			String email, boolean hidden, boolean system, boolean archived) {
		DirEntry ret = new DirEntry();
		ret.orgUnitUid = orgUnitUid;
		ret.path = path;
		ret.kind = kind;
		ret.entryUid = entryUid;
		ret.displayName = displayName;
		ret.email = email;
		ret.hidden = hidden;
		ret.system = system;
		ret.archived = archived;
		return ret;
	}

	public static DirEntry create(String orgUnitUid, String path, Kind kind, String entryUid, String displayName,
			String email, boolean hidden, boolean system, boolean archived, String dataLocation) {
		DirEntry ret = DirEntry.create(orgUnitUid, path, kind, entryUid, displayName, email, hidden, system, archived);
		ret.dataLocation = dataLocation;
		return ret;
	}

	public static DirEntry create(String orgUnitUid, String path, Kind kind, String entryUid, String displayName,
			String email, boolean hidden, boolean system, boolean archived, String dataLocation,
			AccountType accountType) {
		DirEntry ret = DirEntry.create(orgUnitUid, path, kind, entryUid, displayName, email, hidden, system, archived,
				dataLocation);
		ret.accountType = accountType;
		return ret;
	}

	public DirEntry withEmails(String... emails) {
		boolean d = true;
		ArrayList<Email> ret = new ArrayList<>(emails.length);
		for (String e : emails) {
			Email email = Email.create(e, d);
			d = false;
			ret.add(email);
		}
		this.emails = ret;
		return this;
	}

	/**
	 * Adds an array of {@link net.bluemind.core.api.Email}s to this
	 * {@link DirEntry}
	 * 
	 * @param emails array of {@link net.bluemind.core.api.Email}s
	 * @return the {@link DirEntry} itself
	 */

	public DirEntry withEmails(List<Email> emails) {
		this.emails = emails;
		return this;
	}

	@Override
	public String toString() {
		return "DirEntry [kind=" + kind + ", path=" + path + ", displayName=" + displayName + ", entryUid=" + entryUid
				+ ", email=" + email + ", archived = " + archived + "]";
	}
}
