/* BEGIN LICENSE
 * Copyright © Blue Mind SAS, 2012-2016
 *
 * This file is part of BlueMind. BlueMind is a messaging and collaborative
 * solution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU Affero General Public License as
 * published by the Free Software Foundation (version 3 of the License).
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See LICENSE.txt
 * END LICENSE
 */
package net.bluemind.directory.api;

import java.util.Arrays;
import java.util.List;

import net.bluemind.core.api.BMApi;

@BMApi(version = "3")
public class DirEntryQuery {

	public Order order = defaultOrder();

	public String nameFilter;
	public boolean hiddenFilter = true;
	public String emailFilter;
	public String nameOrEmailFilter;
	public StateFilter stateFilter = null;
	public boolean systemFilter = true;

	public List<DirEntry.Kind> kindsFilter;
	public List<String> entries;
	public DirEntry.AccountType accountTypeFilter;

	public int from = 0;
	public int size = -1;

	public List<String> entryUidFilter;
	public boolean onlyManagable = false;

	@BMApi(version = "3")
	public static enum StateFilter {
		Archived, Active, All
	}

	@BMApi(version = "3")
	public static enum OrderBy {
		kind, displayname
	}

	@BMApi(version = "3")
	public static enum Dir {
		asc, desc
	}

	@BMApi(version = "3")
	public static class Order {
		public OrderBy by;
		public Dir dir;
	}

	public static DirEntryQuery all() {
		DirEntryQuery q = new DirEntryQuery();
		return q;
	}

	public static DirEntryQuery entries(List<String> uids) {
		DirEntryQuery q = new DirEntryQuery();
		q.entries = uids;
		return q;
	}

	public static DirEntryQuery entries(String... uid) {
		DirEntryQuery q = new DirEntryQuery();
		q.entries = Arrays.asList(uid);
		return q;
	}

	public static Order defaultOrder() {
		return order(OrderBy.displayname, Dir.asc);
	}

	public static Order order(OrderBy by, Dir dir) {
		Order o = new Order();
		o.by = by;
		o.dir = dir;
		return o;
	}

	public static DirEntryQuery filterKind(DirEntry.Kind... kinds) {
		DirEntryQuery q = new DirEntryQuery();
		q.kindsFilter = Arrays.asList(kinds);
		return q;
	}

	public static DirEntryQuery filterName(String name) {
		DirEntryQuery q = new DirEntryQuery();
		q.nameFilter = name;
		return q;
	}

	public static DirEntryQuery filterEmail(String email) {
		DirEntryQuery q = new DirEntryQuery();
		q.emailFilter = email;
		return q;
	}

	public static DirEntryQuery filterEntryUid(String... entryUids) {
		DirEntryQuery q = new DirEntryQuery();
		q.entryUidFilter = Arrays.asList(entryUids);
		return q;
	}

	public static DirEntryQuery filterNameOrEmail(String string) {
		DirEntryQuery q = new DirEntryQuery();
		q.nameOrEmailFilter = string;
		return q;
	}

}
