/* BEGIN LICENSE
 * Copyright © Blue Mind SAS, 2012-2017
 *
 * This file is part of BlueMind. BlueMind is a messaging and collaborative
 * solution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU Affero General Public License as
 * published by the Free Software Foundation (version 3 of the License).
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See LICENSE.txt
 * END LICENSE
 */
package net.bluemind.directory.api;

import java.util.Set;

import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;

import net.bluemind.core.api.BMApi;
import net.bluemind.core.task.api.TaskRef;

@BMApi(version = "3")
@Path("/directory/{domain}/{entryUid}/mgmt")
public interface IDirEntryMaintenance {

	/**
	 * retrieve available maintenance operations for a dir entry
	 * 
	 * @return
	 */
	@GET
	@Path("_maintenance")
	Set<MaintenanceOperation> getAvailableOperations();

	/**
	 * launch maintenance operations in check mode
	 * 
	 * @param opIdentifiers
	 * @return
	 */
	@POST
	@Path("_maintenance/check")
	TaskRef check(Set<String> opIdentifiers);

	/**
	 * launch maintenance operation in repair mode
	 * 
	 * @param opIdentifiers
	 * @return
	 */
	@POST
	@Path("_maintenance/repair")
	TaskRef repair(Set<String> opIdentifiers);

}
