/* BEGIN LICENSE
 * Copyright © Blue Mind SAS, 2012-2016
 *
 * This file is part of BlueMind. BlueMind is a messaging and collaborative
 * solution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU Affero General Public License as
 * published by the Free Software Foundation (version 3 of the License).
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See LICENSE.txt
 * END LICENSE
 */
package net.bluemind.directory.api;

import java.util.List;
import java.util.Set;

import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.QueryParam;

import net.bluemind.core.api.BMApi;
import net.bluemind.core.container.model.ItemValue;

@BMApi(version = "3")
@Path("/directory/_ou/{domain}")
public interface IOrgUnits {
	/**
	 * Fetch organizational unit by uid
	 * 
	 * @return {@link ItemValue} of ({@link OrgUnit}
	 */
	@GET
	@Path("{uid}/complete")
	public ItemValue<OrgUnit> getComplete(@PathParam(value = "uid") String uid);

	@GET
	@Path("{uid}/_children")
	public List<ItemValue<OrgUnit>> getChildren(@PathParam(value = "uid") String uid);

	/**
	 * Create organizational unit
	 * 
	 * @param uid
	 *                  uid
	 * @param value
	 *                  {@link OrgUnit}
	 * @return {@link ItemValue} of ({@link OrgUnit}
	 */
	@PUT
	@Path("{uid}")
	public void create(@PathParam(value = "uid") String uid, OrgUnit value);

	/**
	 * Update organizational unit
	 * 
	 * @param uid
	 *                  uid
	 * @param value
	 *                  {@link OrgUnit}
	 * @return {@link ItemValue} of ({@link OrgUnit}
	 */
	@POST
	@Path("{uid}")
	public void update(@PathParam(value = "uid") String uid, OrgUnit value);

	/**
	 * Delete organizational unit
	 * 
	 * @param uid
	 *                uid
	 */
	@DELETE
	@Path("{uid}")
	public void delete(@PathParam(value = "uid") String uid);

	/**
	 * Fetch organizational unit path by uid
	 * 
	 * @param uid
	 *                OU uid
	 * @return {@link ItemValue} of ({@link OrgUnitPath}
	 */
	@GET
	@Path("{uid}/path")
	public OrgUnitPath getPath(@PathParam(value = "uid") String uid);

	/**
	 * Search {@link OrgUnitPath}
	 * 
	 * @param query
	 * @return
	 */
	@POST
	@Path("_search")
	public List<OrgUnitPath> search(OrgUnitQuery query);

	@POST
	@Path("{uid}/{dirUid}/_set")
	public void setAdministratorRoles(@PathParam(value = "uid") String uid, @PathParam("dirUid") String dirUid,
			Set<String> roles);

	@POST
	@Path("{uid}/{dirUid}")
	public Set<String> getAdministratorRoles(@PathParam(value = "uid") String uid, @PathParam("dirUid") String dirUid,
			List<String> groups);

	@GET
	@Path("{uid}/_administrators")
	public Set<String> getAdministrators(@PathParam(value = "uid") String uid);

	@POST
	@Path("_byAdmin")
	public List<OrgUnitPath> listByAdministrator(@QueryParam("administrator") String administrator,
			List<String> groups);

	@DELETE
	@Path("_deleteadmin")
	public void removeAdministrator(@QueryParam("administrator") String administrator);

}
